# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2016 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from anyblok.blok import BlokManager
import inspect


def anyblok_init_config(unittest=False):
    from anyblok import config  # noqa import anyblok.config
    from . import config  # noqa import config definition


class AnyBlokPyramidException(Exception):
    pass


def current_blok():
    filename = inspect.stack()[1][1]
    for blok in BlokManager.ordered_bloks:
        if filename.startswith(BlokManager.getPath(blok)):
            return blok

    raise AnyBlokPyramidException("You are not in a Blok")



# -*- coding: utf-8 -*-
# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from zope.sqlalchemy import ZopeTransactionExtension
from zope.sqlalchemy.datamanager import (
    _SESSION_STATE, STATUS_ACTIVE, STATUS_READONLY, STATUS_CHANGED,
    STATUS_INVALIDATED, NO_SAVEPOINT_SUPPORT, _retryable_errors)
import transaction as zope_transaction
from transaction._transaction import Status as ZopeStatus


from zope.interface import implementer
from transaction.interfaces import IDataManagerSavepoint
from anyblok.environment import EnvironmentManager
from sqlalchemy.engine.base import Engine
from sqlalchemy.orm.exc import ConcurrentModificationError
from sqlalchemy.exc import DBAPIError


class AnyBlokSessionDataManager:

    def __init__(self, session, status, transaction_manager,
                 keep_session=False):
        self.transaction_manager = transaction_manager
        self.registry = session._query_cls.registry
        self.transaction = self.registry.session.transaction
        transaction_manager.get().join(self)
        _SESSION_STATE[id(session)] = status
        self.state = 'init'
        self.keep_session = keep_session

    def _finish(self, final_state):
        assert self.transaction is not None
        del _SESSION_STATE[id(self.registry.session)]
        registry = self.registry
        self.transaction = self.registry = None
        self.state = final_state
        if not self.keep_session:
            registry.session.close()
        else:
            registry.session.expire_all()

        EnvironmentManager.set('_precommit_hook', [])

    def abort(self, trans):
        if self.transaction is not None:
            self._finish('aborted')

    def tpc_begin(self, trans):
        self.registry.session.flush()

    def commit(self, trans):
        status = _SESSION_STATE[id(self.registry.session)]
        if status is not STATUS_INVALIDATED:
            if self.registry.session.expire_on_commit:
                self.registry.session.expire_all()
            self._finish('no work')

    def tpc_vote(self, trans):
        if self.transaction is not None:
            self.registry.commit()
            self._finish('committed')

    def tpc_finish(self, trans):
        pass

    def tpc_abort(self, trans):
        assert self.state is not 'committed'

    def sortKey(self):
        return "~AnyBlok:%d" % id(self.transaction)

    @property
    def savepoint(self):
        if set(engine.url.drivername
               for engine in self.transaction._connections.keys()
               if isinstance(engine, Engine)
               ).intersection(NO_SAVEPOINT_SUPPORT):
            raise AttributeError('savepoint')
        return self._savepoint

    def _savepoint(self):
        return AnyBlokSessionSavepoint(self.registry)

    def should_retry(self, error):
        if isinstance(error, ConcurrentModificationError):
            return True
        if isinstance(error, DBAPIError):
            orig = error.orig
            for error_type, test in _retryable_errors:
                if isinstance(orig, error_type):
                    if test is None:
                        return True
                    if test(orig):
                        return True


class AnyBlokTwoPhaseSessionDataManager(AnyBlokSessionDataManager):

    def tpc_vote(self, trans):
        if self.transaction is not None:
            self.transaction.prepare()
            self.state = 'voted'

    def tpc_finish(self, trans):
        if self.transaction is not None:
            self.registry.commit()
            self._finish('committed')

    def tpc_abort(self, trans):
        if self.transaction is not None:
            self.registry.rollback()
            self._finish('aborted commit')

    def sortKey(self):
        # Sort normally
        return "AnyBlok.twophase:%d" % id(self.transaction)


@implementer(IDataManagerSavepoint)
class AnyBlokSessionSavepoint:

    def __init__(self, registry):
        self.registry = registry
        self.transaction = self.registry.session.begin_nested()

    def rollback(self):
        self.transaction.rollback()


def join_transaction(session, initial_state=STATUS_ACTIVE,
                     transaction_manager=zope_transaction.manager,
                     keep_session=False):
    if _SESSION_STATE.get(id(session), None) is None:
        if session.twophase:
            DataManager = AnyBlokTwoPhaseSessionDataManager
        else:
            DataManager = AnyBlokSessionDataManager
        DataManager(session, initial_state, transaction_manager,
                    keep_session=keep_session)


def mark_changed(session, transaction_manager=zope_transaction.manager,
                 keep_session=False):
    session_id = id(session)
    assert (_SESSION_STATE.get(session_id, None) is not STATUS_READONLY,
            "Session already registered as read only")
    join_transaction(session, STATUS_CHANGED, transaction_manager, keep_session)
    _SESSION_STATE[session_id] = STATUS_CHANGED


class AnyBlokZopeTransactionExtension(ZopeTransactionExtension):

    def after_begin(self, session, transaction, connection):
        join_transaction(session, self.initial_state, self.transaction_manager,
                         self.keep_session)

    def after_attach(self, session, instance):
        join_transaction(session, self.initial_state, self.transaction_manager,
                         self.keep_session)

    def after_flush(self, session, flush_context):
        mark_changed(session, self.transaction_manager, self.keep_session)

    def after_bulk_update(self, session, query, query_context, result):
        mark_changed(session, self.transaction_manager, self.keep_session)

    def after_bulk_delete(self, session, query, query_context, result):
        mark_changed(session, self.transaction_manager, self.keep_session)

    def before_commit(self, session):
        assert (session.transaction.nested or
                self.transaction_manager.get().status == ZopeStatus.COMMITTING,
                "Transaction must be committed using the transaction manager")



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2016 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from anyblok.config import Configuration, AnyBlokPlugin
from .release import version
import os


Configuration.applications.update({
    'pyramid': {
        'prog': 'AnyBlok simple wsgi app, version %r' % version,
        'description': "WSGI for test your AnyBlok / Pyramid app",
        'configuration_groups': ['config', 'database'],
    },
    'gunicorn': {
        'prog': 'AnyBlok gunicorn wsgi app, version %r' % version,
        'description': "GUNICORN for test your AnyBlok / Pyramid app",
        'configuration_groups': ['gunicorn', 'database'],
    },
})


def get_db_name(request):
    return Configuration.get('db_name')


@Configuration.add('preload', label="Preload")
def define_preload_option(group):
    group.add_argument('--databases', dest='db_names', nargs="+",
                       help='List of the database allow to be load')


@Configuration.add('wsgi', label="WSGI")
def define_wsgi_option(group):
    group.add_argument(
        '--wsgi-host',
        default=os.environ.get('ANYBLOK_PYRAMID_WSGI_HOST', 'localhost'))
    group.add_argument(
        '--wsgi-port', type=int,
        default=os.environ.get('ANYBLOK_PYRAMID_WSGI_PORT', 5000))


@Configuration.add('pyramid-debug', label="Pyramid")
def define_wsgi_debug_option(group):
    group.add_argument('--pyramid-reload-all', dest='pyramid.reload_all',
                       help="Turns on all reload* settings.",
                       action='store_true')
    group.add_argument('--pyramid-reload-templates',
                       dest='pyramid.reload_templates',
                       help="templates are "
                            "automatically reloaded whenever they are "
                            "modified without restarting the application, so "
                            "you can see changes to templates take effect "
                            "immediately during development. This flag is "
                            "meaningful to Chameleon and Mako templates, as "
                            "well as most third-party template rendering "
                            "extensions.",
                       action='store_true')
    group.add_argument('--pyramid-reload-assets',
                       dest='pyramid.reload_assets',
                       help="Don't cache any asset file data",
                       action='store_true')
    group.add_argument('--pyramid-debug-all',
                       dest='pyramid.debug_all',
                       help="Turns on all debug* settings.",
                       action='store_true')
    group.add_argument('--pyramid-debug-notfound',
                       dest='pyramid.debug_notfound',
                       help="Print view-related NotFound debug messages "
                            "to stderr",
                       action='store_true')
    group.add_argument('--pyramid-debug-routematch',
                       dest='pyramid.debug_routematch',
                       help="Print debugging messages related to url dispatch "
                            "route matching",
                       action='store_true')
    group.add_argument('--pyramid-prevent-http-cache',
                       dest='pyramid.prevent_http_cache',
                       help="Prevent the http_cache view configuration "
                            "argument from having any effect globally in this "
                            "process when this value is true. No http "
                            "caching-related response headers will be set by "
                            "the Pyramid http_cache view configuration "
                            "feature",
                       action='store_true')
    group.add_argument('--pyramid-default-locale-name',
                       dest='pyramid.default_locale_name',
                       help="The value supplied here is used as the default "
                            "locale name when a locale negotiator is not "
                            "registered.",
                       default='')


@Configuration.add('gunicorn')
def add_configuration_file(parser):
    parser.add_argument('--anyblok-configfile', dest='configfile', default='',
                        help="Relative path of the config file")
    parser.add_argument('--without-auto-migration',
                        dest='withoutautomigration',
                        action='store_true')


@Configuration.add('plugins', must_be_loaded_by_unittest=True)
def update_plugins(group):
    group.add_argument('--get-db-name-plugin',
                       dest='get_db_name', type=AnyBlokPlugin,
                       default='anyblok_pyramid.config:get_db_name',
                       help="get_db_name function to use")



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from anyblok.tests.testcase import DBTestCase, LogCapture
from anyblok.config import Configuration
from anyblok_pyramid.common import get_registry_for, preload_databases
from logging import INFO, WARNING


class TestCommon(DBTestCase):

    def test_get_registry_for(self):
        registry = get_registry_for(Configuration.get('db_name'))
        self.assertIsNotNone(registry)

    def test_preload_databases(self):
        db_name = Configuration.get('db_name')
        with DBTestCase.Configuration(db_names=[db_name]):
            with LogCapture('anyblok_pyramid.common', level=INFO) as handler:
                preload_databases()
                messages = handler.get_info_messages()
                self.assertTrue(messages)
                self.assertIn('The database %r is preloaded' % db_name,
                              messages)

    def test_preload_unexisting_databases(self):
        db_name = 'wrong_db_name'
        with DBTestCase.Configuration(db_names=[db_name]):
            with LogCapture('anyblok_pyramid.common', level=WARNING) as handler:
                preload_databases()
                messages = handler.get_warning_messages()
                self.assertTrue(messages)
                self.assertIn('The database %r does not exist' % db_name,
                              messages)



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#    Copyright (C) 2016 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from wsgiref.simple_server import make_server
from anyblok.blok import BlokManager
from anyblok.scripts import format_configuration
from anyblok.config import Configuration
from .pyramid_config import Configurator
import sys
from anyblok import load_init_function_from_entry_points
from .common import preload_databases
from logging import getLogger
logger = getLogger(__name__)


def anyblok_wsgi(application, configuration_groups, **kwargs):
    """
    :param application: name of the application
    :param configuration_groups: list configuration groupe to load
    :param \**kwargs: ArgumentParser named arguments
    """
    format_configuration(configuration_groups, 'preload', 'pyramid-debug',
                         'wsgi')
    load_init_function_from_entry_points()
    Configuration.load(application,
                       configuration_groups=configuration_groups, **kwargs)
    BlokManager.load()
    config = Configurator()
    config.include_from_entry_point()
    config.load_config_bloks()

    wsgi_host = Configuration.get('wsgi_host')
    wsgi_port = int(Configuration.get('wsgi_port'))

    app = config.make_wsgi_app()
    server = make_server(wsgi_host, wsgi_port, app)
    preload_databases()

    logger.info("Serve forever on %r:%r" % (wsgi_host, wsgi_port))
    server.serve_forever()


def wsgi():
    anyblok_wsgi('pyramid', ['logging'])


def gunicorn_anyblok_wsgi(application, configuration_groups, **kwargs):
    try:
        import gunicorn  # noqa
    except ImportError:
        logger.error("No gunicorn installed")
        sys.exit(1)

    format_configuration(configuration_groups, 'preload', 'pyramid-debug')
    from .gunicorn import WSGIApplication
    WSGIApplication(application,
                    configuration_groups=configuration_groups).run()


def gunicorn_wsgi():
    gunicorn_anyblok_wsgi('gunicorn', ['logging'])



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from .testcase import PyramidDBTestCase
from anyblok.config import Configuration
from pyramid.response import Response
from anyblok_pyramid.config import get_db_name


def _get_db_name(request):
    dbname = None
    if request.anyblok.registry:
        dbname = request.anyblok.registry.db_name

    return Response(dbname)


class TestRegistry(PyramidDBTestCase):

    def tearDown(self):
        super(TestRegistry, self).tearDown()
        Configuration.update(get_db_name=get_db_name)

    def add_route_and_views(self, config):
        config.add_route('dbname', '/test/')
        config.add_view(_get_db_name, route_name='dbname')

    def test_registry_by_default_method(self):
        self.includemes.append(self.add_route_and_views)
        webserver = self.init_web_server()
        res = webserver.get('/test/', status=200)
        self.assertEqual(Configuration.get('db_name'), res.body.decode('utf8'))

    def test_registry_by_wrong_name(self):
        def __get_db_name(request):
            return 'wrong_db_name'

        Configuration.update(get_db_name=__get_db_name)
        self.includemes.append(self.add_route_and_views)
        webserver = self.init_web_server()
        res = webserver.get('/test/', status=200)
        self.assertFalse(res.body)

    def test_registry_by_path(self):
        def __get_db_name(request):
            return request.matchdict['dbname']

        def add_route_and_views(config):
            config.add_route('dbname', '/test/{dbname}/')
            config.add_view(_get_db_name, route_name='dbname')

        Configuration.update(get_db_name=__get_db_name)
        self.includemes.append(add_route_and_views)
        webserver = self.init_web_server()
        res = webserver.get('/test/%s/' % Configuration.get('db_name'),
                            status=200)
        self.assertEqual(Configuration.get('db_name'), res.body.decode('utf8'))
        res = webserver.get('/test/wrong_db_name/', status=200)
        self.assertFalse(res.body)



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2016 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from os.path import join
from pyramid.config import Configurator as PConfigurator
from anyblok.blok import BlokManager
from anyblok.config import Configuration
from pkg_resources import iter_entry_points
from .common import get_registry_for
from logging import getLogger
logger = getLogger(__name__)


class AnyBlokRequest:
    """ Add anyblok properties in the request
    ::

        request.anyblok

    """
    def __init__(self, request):
        self.request = request

    @property
    def registry(self):
        """ Add the property registry
        ::

            registry = request.anyblok.registry

        .. note::

            The db_name must be defined

        """
        dbname = Configuration.get('get_db_name')(self.request)
        if Configuration.get('Registry').db_exists(db_name=dbname):
            return get_registry_for(dbname)
        else:
            return None


class NeedAnyBlokRegistryPredicate:
    """ Predicate ``need_anyblok_registry`` """

    def __init__(self, need_anyblok_registry, config):
        self.need_anyblok_registry = need_anyblok_registry

    def text(self):
        return 'Need AnyBlok registry = %s' % str(self.need_anyblok_registry)

    phash = text

    def __call__(self, context, request):
        if self.need_anyblok_registry:
            if not request.anyblok:
                return False

            if not request.anyblok.registry:
                return False

        return True


class InstalledBlokPredicate:
    """ Predicate ``installed_blok`` """

    def __init__(self, blok_name, config):
        self.blok_name = blok_name

    def text(self):
        return 'instaled blok = %s' % self.blok_name

    phash = text

    def __call__(self, context, request):
        if not request.anyblok:
            return False

        if not request.anyblok.registry:
            return False

        # use this method because she is cached
        return request.anyblok.registry.System.Blok.is_installed(
            self.blok_name)


class Configurator(PConfigurator):
    """Overwrite the Pyramid Configurator"""

    def __init__(self, *args, **kwargs):
        kwargs = self.default_kwargs(**kwargs)
        super(Configurator, self).__init__(*args, **kwargs)

    def default_kwargs(self, **kwargs):
        if 'settings' not in kwargs:
            kwargs['settings'] = self.default_setting()

        return kwargs

    def default_setting(self):
        """Call all the entry point ``anyblok_pyramid.settings`` to update
        the argument setting

        the callable need to have one parametter, it is a dict::

            def settings_callable(setting):
                ...

        We add the entry point by the setup file::

            setup(
                ...,
                entry_points={
                    'anyblok_pyramid.settings': [
                        settings_callable=path:settings_callable,
                        ...
                    ],
                },
                ...,
            )


        """
        settings = {}
        for i in iter_entry_points('anyblok_pyramid.settings'):
            logger.debug('Load settings: %r' % i.name)
            i.load()(settings)

        return settings

    def include_from_entry_point(self):
        """Call all the entry point ``anyblok_pyramid.includeme`` to update
        the pyramid configuration

        the callable need to have one parametter(the instance of
        ``Configurator`` class, self)::

            def config_callable(config):
                config.include(...)

        We add the entry point by the setup file::

            setup(
                ...,
                entry_points={
                    'anyblok_pyramid.includeme': [
                        config_callable=path:config_callable,
                        ...
                    ],
                },
                ...,
            )


        """
        self.add_request_method(AnyBlokRequest, 'anyblok', reify=True)
        self.add_route_predicate('installed_blok', InstalledBlokPredicate)
        self.add_view_predicate('installed_blok', InstalledBlokPredicate)
        self.add_route_predicate('need_anyblok_registry',
                                 NeedAnyBlokRegistryPredicate)
        self.add_view_predicate('need_anyblok_registry',
                                NeedAnyBlokRegistryPredicate)
        for i in iter_entry_points('anyblok_pyramid.includeme'):
            logger.debug('Load includeme: %r' % i.name)
            i.load()(self)

    def load_config_bloks(self):
        """ loop on each blok, keep the order of the blok to load the
        pyramid config. The blok must declare the meth
        ``pyramid_load_config``::

            def pyramid_load_config(config):
                config.add_route('hello', '/hello/{name}/')
                ...

        """
        self.commit()
        for blok_name in BlokManager.ordered_bloks:
            blok = BlokManager.get(blok_name)
            if hasattr(blok, 'pyramid_load_config'):
                logger.debug('Load configuration from: %r' % blok_name)
                blok.pyramid_load_config(self)
                self.commit()


def pyramid_settings(settings):
    """Add in settings the default value for pyramid configuration

    :param settings: dict of the existing settings
    """
    settings.update({
        'pyramid.reload_templates': Configuration.get(
            'pyramid.reload_templates'),
        'pyramid.reload_assets': Configuration.get('pyramid.reload_assets'),
        'pyramid.debug_notfound': Configuration.get(
            'pyramid.debug_notfound'),
        'pyramid.debug_routematch': Configuration.get(
            'pyramid.debug_routematch'),
        'pyramid.prevent_http_cache': Configuration.get(
            'pyramid.prevent_http_cache'),
        'pyramid.debug_all': Configuration.get('pyramid.debug_all'),
        'pyramid.reload_all': Configuration.get('pyramid.reload_all'),
        'pyramid.default_locale_name': Configuration.get(
            'pyramid.default_locale_name'),
    })


def pyramid_tm(config):
    """Add beaker includeme in pyramid configuration

    :param config: Pyramid configurator instance
    """

    config.include('pyramid_tm')


def static_paths(config):
    """Pyramid includeme, add the static path of the blok

    :param config: Pyramid configurator instance
    """

    for blok, cls in BlokManager.bloks.items():
        if hasattr(cls, 'static_paths'):
            paths = cls.static_paths
            if isinstance(paths, str):
                paths = [paths]
        else:
            paths = ['static']

        blok_path = BlokManager.getPath(blok)

        for p in paths:
            config.add_static_view(join(blok, p), join(blok_path, p))



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from anyblok_pyramid.config import (define_preload_option,
                                    define_wsgi_option,
                                    define_wsgi_debug_option,
                                    add_configuration_file,
                                    update_plugins)
from anyblok.tests.testcase import TestCase
from anyblok.tests.test_config import MockArgumentParser


class TestArgsParseOption(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TestArgsParseOption, cls).setUpClass()
        cls.parser = MockArgumentParser()
        cls.group = cls.parser.add_argument_group('label')
        cls.configuration = {}
        cls.function = {
            'define_preload_option': define_preload_option,
            'define_wsgi_option': define_wsgi_option,
            'define_wsgi_debug_option': define_wsgi_debug_option,
            'add_configuration_file': add_configuration_file,
            'update_plugins': update_plugins,
        }

    def test_define_preload_option(self):
        self.function['define_preload_option'](self.parser)

    def test_define_wsgi_option(self):
        self.function['define_wsgi_option'](self.parser)

    def test_define_wsgi_debug_option(self):
        self.function['define_wsgi_debug_option'](self.parser)

    def test_add_configuration_file(self):
        self.function['add_configuration_file'](self.parser)

    def test_update_plugins(self):
        self.function['update_plugins'](self.parser)



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from setuptools import setup, find_packages
import os
version = '0.7.1'

requires = [
    'anyblok>=0.9.0',
    'pyramid',
    'pyramid_tm',
    'zope.sqlalchemy',
]

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst'), 'r', encoding='utf-8') as readme:
    README = readme.read()

with open(
    os.path.join(here, 'doc', 'FRONT.rst'), 'r', encoding='utf-8'
) as front:
    FRONT = front.read()

with open(
    os.path.join(here, 'doc', 'CHANGES.rst'), 'r', encoding='utf-8'
) as change:
    CHANGE = change.read()

console_scripts = [
    'anyblok_pyramid=anyblok_pyramid.scripts:wsgi',
    'gunicorn_anyblok_pyramid=anyblok_pyramid.scripts:gunicorn_wsgi',
]

anyblok_pyramid_includeme = [
    'pyramid_tm=anyblok_pyramid.pyramid_config:pyramid_tm',
    'static_paths=anyblok_pyramid.pyramid_config:static_paths',
]
anyblok_init = [
    'anyblok_pyramid_config=anyblok_pyramid:anyblok_init_config',
]

setup(
    name="anyblok_pyramid",
    version=version,
    author="Jean-Sébastien Suzanne",
    author_email="jssuzanne@anybox.fr",
    description="Web Server Pyramid for AnyBlok",
    license="MPL2",
    long_description=README + '\n' + FRONT + '\n' + CHANGE,
    url="http://docs.anyblok-pyramid.anyblok.org/" + version,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=requires,
    tests_require=requires + ['nose', 'WebTest'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Framework :: Pyramid',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
        'Topic :: Internet :: WWW/HTTP :: Session',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Server',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
    ],
    entry_points={
        'console_scripts': console_scripts,
        'anyblok_pyramid.settings': [
            'pyramid_settings=anyblok_pyramid.pyramid_config:pyramid_settings',
        ],
        'anyblok_pyramid.includeme': anyblok_pyramid_includeme,
        'anyblok.init': anyblok_init,
        'test_bloks': [
            'test-pyramid-blok1=anyblok_pyramid.test_bloks.test_pyramid_blok1:'
            'TestPyramidBlok',
        ]
    },
    extras_require={},
)



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from anyblok.tests.testcase import DBTestCase, BlokTestCase
from webtest import TestApp
from ..pyramid_config import Configurator


class PyramidTestCase:

    def setUp(self):
        super(PyramidTestCase, self).setUp()
        self.includemes = []

    def init_web_server(self):
        config = Configurator()
        config.include_from_entry_point()
        for includeme in self.includemes:
            config.include(includeme)

        config.load_config_bloks()
        app = config.make_wsgi_app()
        return TestApp(app)


class PyramidDBTestCase(PyramidTestCase, DBTestCase):

    def init_registry(self, function, **kwargs):
        res = super(PyramidDBTestCase, self).init_registry(function, **kwargs)
        self.webserver = self.init_web_server()
        return res


class PyramidBlokTestCase(PyramidTestCase, BlokTestCase):

    @property
    def webserver(self):
        return self.init_web_server()
